package edu.pacificu.cs.cs360.PingPong;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.Set;

public class PingPongServer
{

	private ServerSocketChannel serverChannel;
	private Selector selector;


	/**
	 * Instantiates a new ping pong server.
	 * 
	 * @param port
	 *          the port
	 */
	public PingPongServer(int port)
	{
		try
		{

			serverChannel = ServerSocketChannel.open();
			ServerSocket ss = serverChannel.socket();
			InetSocketAddress address = new InetSocketAddress(port);
			ss.bind(address);
			serverChannel.configureBlocking(false);
			selector = Selector.open();
			serverChannel.register(selector, SelectionKey.OP_ACCEPT);

		}
		catch (IOException ex)
		{
			ex.printStackTrace();
			return;
		}

	}

	/**
	 * Run the server. This method loops forever and handles events produced by
	 * the selector.
	 * 
	 */
	public void run()
	{

		boolean running = true;
		while (running)
		{

			/*
			 * Grab keys from the the Selector
			 */
			try
			{
				selector.select(100);
			}
			catch (IOException ex)
			{
				ex.printStackTrace();
				running = false;
			}

			Set<SelectionKey> readyKeys = selector.selectedKeys();
			java.util.Iterator<SelectionKey> iterator = readyKeys.iterator();

			while (iterator.hasNext())
			{

				SelectionKey key = (SelectionKey) iterator.next();
				iterator.remove();

				try
				{

					/* accept a connection */
					if (key.isAcceptable())
					{
						ServerSocketChannel server = (ServerSocketChannel) key.channel();
						SocketChannel client = server.accept();
						System.out.println("Accepted connection from " + client);

					}
					else if (key.isWritable())
					{

						/* this connection is writable */

					}
					else if (key.isReadable())
					{
						/* this connection is readable */

					}

				}
				catch (IOException ex)
				{

					running = false;
					key.cancel();

					try
					{
						key.channel().close();
					}
					catch (IOException cex)
					{
						
					}
				}

			}
		}

	}

	/**
	 * The main method.
	 * 
	 * @param args
	 *          If any argument is given, that argument is used as the listening
	 *          port
	 */
	public static void main(String[] args)
	{
		int port = 12348;

		if (1 == args.length)
		{
			port = Integer.valueOf(args[0]);

		}

		PingPongServer theServer = new PingPongServer(port);
		theServer.run();
	}
}
